# accounts.py
import requests
from datetime import datetime, timedelta

from .auth import TelematicsAuth
from .core import TelematicsCore
from .utility import handle_response, adjust_date_range
from requests.exceptions import HTTPError, JSONDecodeError
import json




class BaseEngament:
    ACCOUNTS_URL = "https://accounts.telematicssdk.com"

    
    def __init__(self, auth_client: TelematicsAuth):
        self.auth_client = auth_client
    
    def _get_headers(self):
        return {
            'accept': 'application/json',
            'authorization': f'Bearer {self.auth_client.get_access_token()}'
        }
        
class AccountsModule:
    def __init__(self, core: TelematicsCore):
        self.core = core  # Renamed self.code to self.core for clarity
        
    @property
    def Accounts(self):
        return Accounts(self.core.auth_client)

class AccountsResponse:
    def __init__(self, data):
        # Check if data is None or empty before assignment
        if data is None or not data:
            self.data = {}
        else:
            self.data = data

    @property
    def result(self):
        return self.data.get('Result', {})

    @property
    def status(self):
        return self.data.get('Status', {})

    @property
    def errors(self):
        """
        Extracts and returns error messages if they exist in the response.
        """
        error_messages = []
        if 'Errors' in self.data:
            for error in self.data['Errors']:
                message = error.get('Message')
                if message:
                    error_messages.append(message)
        return error_messages
    
    def __iter__(self):
        for item in self.data:
            yield item

    def __str__(self):
        return json.dumps(self.data, indent=4)
    

class Accounts(BaseEngament):
    def create_application(self, company_id, name, description, status=1, 
                           googlePlayLink='', appleStoreLink='', createDefaultGroup=True):
        """
        Create an application for a specified company.

        :param company_id: The ID of the company.
        :param name: Name of the application.
        :param description: Description of the application.
        :param status: Status of the application (default is 1).
        :param googlePlayLink: Link to the application on Google Play (default is empty string).
        :param appleStoreLink: Link to the application on Apple Store (default is empty string).
        :param createDefaultGroup: Flag to create a default group (default is True).
        :return: An AccountsResponse object with the result or None.
        """
        url = f"{self.ACCOUNTS_URL}/v1/companies/{company_id}/applications"
        headers = self._get_headers()
        headers['Content-Type'] = 'application/json-patch+json'
        headers['Accept'] = 'application/json'

        app_data = {
            "name": name,
            "description": description,
            "status": status,
            "googlePlayLink": googlePlayLink,
            "appleStoreLink": appleStoreLink,
            "createDefaultGroup": createDefaultGroup
        }

        try:
            response = self.auth_client.post_with_retry(url, headers=headers, json=app_data)
            data = handle_response(response)
            if data is not None:
                return AccountsResponse(data)
            return None
        except HTTPError as http_err:
            print(f'HTTP error occurred: {http_err}')  # Or handle it in some other way
            e_response = handle_response(response, AccountsResponse)  # Pass AccountsResponse as an argument
            return e_response

        
    def get_applications(self, company_id):
        """
        Retrieve a list of applications for a specified company.

        :param company_id: The ID of the company.
        :return: A response object containing the list of applications.
        """
        url = f"{self.ACCOUNTS_URL}/v1/companies/{company_id}/applications"
        headers = self._get_headers()

        try:
            response = self.auth_client.get_with_retry(url, headers=headers)
            data = handle_response(response)
            if data is not None:
                return AccountsResponse(data)
            return None
        except HTTPError as http_err:
            print(f'HTTP error occurred: {http_err}')  # Or handle it in some other way
            e_response = handle_response(response, AccountsResponse)  # Pass AccountsResponse as an argument
            return e_response

    
    def get_application_details(self, app_id, include_instances=False):
        """
        Fetch details of a specific application.

        :param app_id: The ID of the application.
        :param include_instances: Flag to include instances in the response (default is False).
        :return: A response object containing the application details.
        """
        url = f"{self.ACCOUNTS_URL}/v1/Applications/{app_id}?includeinstances={str(include_instances).lower()}"
        headers = self._get_headers()

        try:
            response = self.auth_client.get_with_retry(url, headers=headers)
            data = handle_response(response)
            if data is not None:
                return AccountsResponse(data)
            return None
        except HTTPError as http_err:
            print(f'HTTP error occurred: {http_err}')  # Or handle it in some other way
            e_response = handle_response(response, AccountsResponse)  # Pass AccountsResponse as an argument
            return e_response


    def update_application_details(self, app_id, status, name, description, google_play_link='', apple_store_link=''):
        """
        Update details of a specific application.

        :param app_id: The ID of the application.
        :param status: Status of the application.
        :param name: Name of the application.
        :param description: Description of the application.
        :param google_play_link: Google Play link of the application (optional).
        :param apple_store_link: Apple Store link of the application (optional).
        :return: A response object indicating the result of the update operation.
        """
        url = f"{self.ACCOUNTS_URL}/v1/Applications/applications/{app_id}"
        headers = self._get_headers()
        headers['content-type'] = 'application/json'
        data = {
            "status": status,
            "name": name,
            "description": description,
            "googlePlayLink": google_play_link,
            "appleStoreLink": apple_store_link
        }
        try:
            response = self.auth_client.patch_with_retry(url, headers=headers)
            data = handle_response(response)
            if data is not None:
                return AccountsResponse(data)
            return None
        except HTTPError as http_err:
            print(f'HTTP error occurred: {http_err}')  # Or handle it in some other way
            e_response = handle_response(response, AccountsResponse)  # Pass AccountsResponse as an argument
            return e_response

    def delete_application(self, app_id):
        """
        Delete a specific application.

        :param app_id: The ID of the application to be deleted.
        :return: A response object indicating the result of the delete operation.
        """
        url = f"{self.ACCOUNTS_URL}/v1/Applications/applications/{app_id}"
        headers = self._get_headers()
        try:
            response = self.auth_client.delete_with_retry(url, headers=headers)
            data = handle_response(response)
            if data is not None:
                return AccountsResponse(data)
            return None
        except HTTPError as http_err:
            print(f'HTTP error occurred: {http_err}')  # Or handle it in some other way
            e_response = handle_response(response, AccountsResponse)  # Pass AccountsResponse as an argument
            return e_response


    def create_instance(self, app_id, name, description, status=1):
        """
        Create an instance for a specified application.

        :param app_id: The ID of the application.
        :param name: Name of the instance.
        :param description: Description of the instance.
        :param status: Status of the instance (default is 1).
        :return: A response object indicating the result of the create operation.
        """
        url = f"{self.ACCOUNTS_URL}/v1/applications/{app_id}/instances"
        headers = self._get_headers()
        headers['content-type'] = 'application/json'
        instance_data = {
            "status": status,
            "name": name,
            "description": description
        }

        try:
            response = self.auth_client.post_with_retry(url, headers=headers, json=instance_data)
            data = handle_response(response)
            if data is not None:
                return AccountsResponse(data)
            return None
        except HTTPError as http_err:
            print(f'HTTP error occurred: {http_err}')
            e_response = handle_response(response, AccountsResponse)
            return e_response
        
    
    def get_instances(self, app_id, include_deactivated=False):
        """
        Retrieve a list of instances for a specified application.

        :param app_id: The ID of the application.
        :param include_deactivated: Flag to include deactivated instances in the response (default is False).
        :return: A response object containing the list of instances.
        """
        url = f"{self.ACCOUNTS_URL}/v1/applications/{app_id}/instances?includeDeactivated={str(include_deactivated).lower()}"
        headers = self._get_headers()

        try:
            response = self.auth_client.get_with_retry(url, headers=headers)
            data = handle_response(response)
            if data is not None:
                return AccountsResponse(data)
            return None
        except HTTPError as http_err:
            print(f'HTTP error occurred: {http_err}')
            e_response = handle_response(response, AccountsResponse)
            return e_response
        
    def get_instance_details(self, instance_id):
        """
        Fetch details of a specific instance.

        :param instance_id: The ID of the instance.
        :return: A response object containing the instance details.
        """
        url = f"{self.ACCOUNTS_URL}/v1/Instances/{instance_id}"
        headers = self._get_headers()

        try:
            response = self.auth_client.get_with_retry(url, headers=headers)
            data = handle_response(response)
            if data is not None:
                return AccountsResponse(data)
            return None
        except HTTPError as http_err:
            print(f'HTTP error occurred: {http_err}')
            e_response = handle_response(response, AccountsResponse)
            return e_response

    def get_instance_details_by_invite_code(self, app_id, invite_code):
        """
        Retrieve instance details by invite code for a specific application.

        :param app_id: The ID of the application.
        :param invite_code: The invite code of the instance.
        :return: A response object containing the instance details.
        """
        url = f"{self.ACCOUNTS_URL}/v1/applications/{app_id}/instances/{invite_code}"
        headers = self._get_headers()

        try:
            response = self.auth_client.get_with_retry(url, headers=headers)
            data = handle_response(response)
            if data is not None:
                return AccountsResponse(data)
            return None
        except HTTPError as http_err:
            print(f'HTTP error occurred: {http_err}')  # Or handle it in some other way
            e_response = handle_response(response, AccountsResponse)  # Pass AccountsResponse as an argument
            return e_response


    def update_instance_details(self, instance_id, name, description, status):
        """
        Update details of a specific instance.

        :param instance_id: The ID of the instance.
        :param name: Name of the instance.
        :param description: Description of the instance.
        :param status: Status of the instance.
        :return: A response object indicating the result of the update operation.
        """
        url = f"{self.ACCOUNTS_URL}/v1/Instances/{instance_id}?Name={name}&Description={description}&Status={status}"
        headers = self._get_headers()

        try:
            response = self.auth_client.patch_with_retry(url, headers=headers)
            data = handle_response(response)
            if data is not None:
                return AccountsResponse(data)
            return None
        except HTTPError as http_err:
            print(f'HTTP error occurred: {http_err}')  # Or handle it in some other way
            e_response = handle_response(response, AccountsResponse)  # Pass AccountsResponse as an argument
            return e_response

    def delete_instance(self, instance_id):
        """
        Delete a specific instance.

        :param instance_id: The ID of the instance to be deleted.
        :return: A response object indicating the result of the delete operation.
        """
        url = f"{self.ACCOUNTS_URL}/v1/Instances/{instance_id}"
        headers = self._get_headers()

        try:
            response = self.auth_client.delete_with_retry(url, headers=headers)
            data = handle_response(response)
            if data is not None:
                return AccountsResponse(data)
            return None
        except HTTPError as http_err:
            print(f'HTTP error occurred: {http_err}')  # Or handle it in some other way
            e_response = handle_response(response, AccountsResponse)  # Pass AccountsResponse as an argument
            return e_response


    
def DamoovAuth(email, password):
    auth_client = TelematicsAuth(email, password)
    return Accounts(auth_client)


# engagement.py
import requests
from datetime import datetime, timedelta

from .auth import TelematicsAuth
from .core import TelematicsCore
from .utility import handle_response, adjust_date_range
from requests.exceptions import HTTPError, JSONDecodeError
import json




class BaseEngament:
    # BASE_URL = "https://api.telematicssdk.com/indicators/admin/v2"
    LEADERBOARD_URL = "https://leaderboard.telematicssdk.com/v1/Leaderboard"

    
    def __init__(self, auth_client: TelematicsAuth):
        self.auth_client = auth_client
    
    def _get_headers(self):
        return {
            'accept': 'application/json',
            'authorization': f'Bearer {self.auth_client.get_access_token()}'
        }
        
class EngagementModule:
    def __init__(self, core: TelematicsCore):
        self.core = core  # Renamed self.code to self.core for clarity
        
    @property
    def Engagement(self):
        return Engagement(self.core.auth_client)



class EngagementResponse:
    def __init__(self, data):
        # Check if data is None or empty before assignment
        if data is None or not data:
            self.data = {}
        else:
            self.data = data

    @property
    def result(self):
        results = self.data.get('Result',{})
        return results

    @property
    def status(self):
        return self.data.get('Status',{})

    def __iter__(self):
        for item in self.data:
            yield item


    def __str__(self):
        return json.dumps(self.data, indent=4)
    

    

class Engagement(BaseEngament):
    def get_user_leaderboard(self, user_id):
        headers = {
            'Devicetoken': user_id,
            'accept': 'application/json',
        }

        url = f"{self.LEADERBOARD_URL}/user"

        try:
            response = requests.get(url, headers=headers)
            response.raise_for_status()
            try:
                return EngagementResponse(response.json())
            except JSONDecodeError:
                return EngagementResponse({})  # Return empty data in case of JSONDecodeError

        except HTTPError as http_err:
            print(f'HTTP error occurred: {http_err}')
            e_response = handle_response(response, EngagementResponse)
            return e_response

    def get_general_leaderboard(self, user_id, leaders_count=5, round_users_count=2, ratingtype=1):
        headers = {
            'DeviceToken': user_id,
            'accept': 'application/json',
        }

        params = {
            'UsersCount': leaders_count,
            'RoundUsersCount': round_users_count,
            'Scoringrate': ratingtype
        }

        url = f"{self.LEADERBOARD_URL}"

        try:
            response = requests.get(url, headers=headers, params=params)
            response.raise_for_status()
            try:
                return EngagementResponse(response.json())
            except JSONDecodeError:
                return EngagementResponse({})  # Return empty data in case of JSONDecodeError

        except HTTPError as http_err:
            print(f'HTTP error occurred: {http_err}')
            e_response = handle_response(response, EngagementResponse)
            return e_response



    
        
def DamoovAuth(email, password):
    auth_client = TelematicsAuth(email, password)
    return Engagement(auth_client)


# users.py
import requests
from datetime import datetime, timedelta

from .auth import TelematicsAuth
from .core import TelematicsCore
from .utility import handle_response
import json
from requests.exceptions import HTTPError




class BaseUsers:
    BASE_URL = "https://user.telematicssdk.com/v1"
    
    def __init__(self, auth_client: TelematicsAuth):
        self.auth_client = auth_client
    
    def _get_headers(self):
        return {
            'accept': 'application/json',
            'authorization': f'Bearer {self.auth_client.get_access_token()}'
        }
        
class UsersModule:
    def __init__(self, core: TelematicsCore):
        self.core = core  # Renamed self.code to self.core for clarity
        
    @property
    def Users(self):
        return Users(self.core.auth_client)



class Users(BaseUsers):
    def create_user(self, 
                    instanceid,
                    instancekey,
                    FirstName=None,
                    LastName=None,
                    Nickname=None,
                    Phone=None,
                    Email=None,
                    ClientId=None,
                    CreateAccessToken= False
                    ):
        
        # Parameter validation
        if not instanceid or not instancekey:
            raise ValueError("Both `instanceid` and `instancekey` must be provided.")
   
        
        headers = {
            'InstanceId': instanceid,
            'InstanceKey': instancekey,
            'accept': 'application/json',
            'content-type': 'application/json'
        }
        
        payload={}
        payload["CreateAccessToken"] = CreateAccessToken
        
        if ClientId:
            payload["UserFields"] = {"ClientId": ClientId}
            
            
        self._add_to_payload_if_exists(payload, "FirstName", FirstName)
        self._add_to_payload_if_exists(payload, "LastName", LastName)
        self._add_to_payload_if_exists(payload, "Nickname", Nickname)
        self._add_to_payload_if_exists(payload, "Phone", Phone)
        self._add_to_payload_if_exists(payload, "Email", Email)

        
        url = f"{self.BASE_URL}/registration/create"
        try:
            response = self.auth_client.post_with_retry(url, headers=headers, data=json.dumps(payload))
            
            processed_response = handle_response(response)
            return UsersResponse(processed_response)
    
        except HTTPError as http_err:
            print(f'HTTP error occurred: {http_err}')  # Or handle it in some other way
            handle_response(response)
            return response
        
    def update_user(self, 
                    userid,
                    ClientId=None,
                    FirstName=None,
                    LastName=None,
                    Nickname=None,
                    Phone=None,
                    Email=None
                    ):
        
        headers = {
            'UserDeviceToken': userid,
            'authorization': f'Bearer {self.auth_client.get_access_token()}',
            'accept': 'application/json',
            'content-type': 'application/json'
        }

        payload = {}

        if ClientId:
            payload["UserFields"] = {"ClientId": ClientId}

        self._add_to_payload_if_exists(payload, "FirstName", FirstName)
        self._add_to_payload_if_exists(payload, "LastName", LastName)
        self._add_to_payload_if_exists(payload, "Nickname", Nickname)
        self._add_to_payload_if_exists(payload, "Phone", Phone)
        self._add_to_payload_if_exists(payload, "Email", Email)

        url = f"{self.BASE_URL}/Management/users"
        try:
            response = self.auth_client.put_with_retry(url, headers=headers, data=json.dumps(payload))
            processed_response = handle_response(response)
            return UsersResponse(response)
        except HTTPError as http_err:
            print(f'HTTP error occurred: {http_err}')  # Or handle it in some other way
            handle_response(response)
            return response

    def delete_user(self, userid):
        headers = {
            'accept': 'application/json',
            'Authorization': f'Bearer {self.auth_client.get_access_token()}'
        }
        
        url = f"{self.BASE_URL}/Management/users/{userid}"
        try:
            response = self.auth_client.delete_with_retry(url, headers=headers)
            return UsersResponse(response)
        except HTTPError as http_err:
            print(f'HTTP error occurred: {http_err}')  # Or handle it in some other way
            handle_response(response)
            return response
    
    def get_user_details(self, userid):
        """
        Retrieves details for a specific user based on their device token.

        :param devicetoken: The device token of the user.
        :return: User details in JSON format.
        """
        headers = {
            'accept': 'application/json',
            'authorization': f'Bearer {self.auth_client.get_access_token()}'
        }

        url = f"{self.BASE_URL}/Management/users/find?DeviceToken={userid}"

        try:
            response = self.auth_client.get_with_retry(url, headers=headers)
            response.raise_for_status()  # Raises an error for 4xx and 5xx responses
            return UsersResponse(response.json())
        except HTTPError as http_err:
            print(f'HTTP error occurred: {http_err}')  # Or handle it in some other way
            handle_response(response)
            return response
        

    def _add_to_payload_if_exists(self, payload, key, value):
        if value:
            payload[key] = value

    def update_user_config(self, userid, tracking_enabled=None, api_enabled=None, log_enabled=None):
        """
        Update tracking status for a user.

        :param userid: User ID to update.
        :param tracking_enabled: Boolean to enable/disable tracking.
        :param api_enabled: Boolean to enable/disable API.
        :param log_enabled: Boolean to enable/disable logging.
        :return: UsersResponse object with the result of the update operation.
        """
        # Validate that parameters are either boolean or None
        for param in [tracking_enabled, api_enabled, log_enabled]:
            if param is not None and not isinstance(param, bool):
                raise ValueError("Parameters tracking_enabled, api_enabled, and log_enabled must be boolean or None.")

        headers = {
            'UserDeviceToken': userid,
            'authorization': f'Bearer {self.auth_client.get_access_token()}',
            'accept': 'application/json',
            'content-type': 'application/json'
        }

        payload = {"UserFields": {}}

        if tracking_enabled is not None:
            payload["UserFields"]["EnableTracking"] = tracking_enabled
        if api_enabled is not None:
            payload["UserFields"]["Enabled"] = api_enabled
        if log_enabled is not None:
            payload["UserFields"]["EnableLogging"] = log_enabled

        url = f"{self.BASE_URL}/Management/users"
        try:
            response = self.auth_client.put_with_retry(url, headers=headers, data=json.dumps(payload))
            return UsersResponse(response)
        except HTTPError as http_err:

            return handle_response(http_err)

    def change_user_instance(self, deviceToken, ToInstanceId, ToInstanceKey):
        """
        Change the instance of a user.

        :param deviceToken: The device token of the user.
        :param ToInstanceId: The ID of the instance to change to.
        :param ToInstanceKey: The key of the instance to change to.
        :return: Response from the server.
        """
        headers = {
            'accept': 'application/json',
            'Authorization': f'Bearer {self.auth_client.get_access_token()}',
            'Content-Type': 'application/json-patch+json'
        }

        payload = {
            "DeviceToken": deviceToken,
            "ToInstanceId": ToInstanceId,
            "ToInstanceKey": ToInstanceKey
        }

        url = f"{self.BASE_URL}/Management/users/instances/change"

        try:
            response = self.auth_client.post_with_retry(url, headers=headers, data=json.dumps(payload))
            return UsersResponse(response)
        except HTTPError as http_err:
            print(f'HTTP error occurred: {http_err}')  # Or handle it in some other way
            handle_response(response)
            return response

    
class UsersResponse:
    def __init__(self, input_data):
        # Check if input_data is None
        if input_data is None:
            self.data = {}  # default to an empty dictionary
        # If input_data is an instance of a dictionary, use it directly
        elif isinstance(input_data, dict):
            self.data = input_data
        # If it's something else (like a response object), try to parse its JSON content
        else:
            try:
                self.data = input_data.json()
            except (ValueError, AttributeError):
                self.data = {}  # Handle cases where the response isn't JSON or input_data has no json() method

    @property
    def result(self):
        result = self.data.get('Result', None)
        if isinstance(result, list):
            return result
        elif isinstance(result, dict):
            return [result]  # Wrap the dictionary in a list
        return []  # Default to an empty list if 'Result' is neither a list nor a dictionary
    
    
    @property
    def devicetoken(self):
        results = self.data.get('Result',{}).get('DeviceToken', None)
        return results
    
    @property
    def accesstoken(self):
        # Access the AccessToken and its subfields, with default values if not present
        access_token_info = self.data.get('Result', {}).get('AccessToken', {}).get("Token", None)
        return access_token_info

    @property
    def refreshtoken(self):
        # Retrieve the RefreshToken, with a default value if not present
        return self.data.get('Result', {}).get('RefreshToken', None)



    @property
    def userid(self):
        results = self.data.get('Result',{}).get('DeviceToken', [])
        return results

    @property
    def status(self):
        return self.data.get('Status',{})

    def __iter__(self):
        for item in self.data:
            yield item

    def __str__(self):
        return json.dumps(self.data, indent=4)
    

    
    
        # Add at the bottom of statistics.py
def DamoovAuth(email, password):
    auth_client = TelematicsAuth(email, password)
    return Users(auth_client)





# trips.py

import requests
from datetime import datetime, timedelta

from .auth import TelematicsAuth
from .core import TelematicsCore
from .utility import handle_response, adjust_date_range
import json
from requests.exceptions import HTTPError



class BaseTrips:
    BASE_URL = "https://api.telematicssdk.com/trips/get/admin/v1"

    def __init__(self, auth_client: TelematicsAuth):
        self.auth_client = auth_client

    def _get_headers(self):
        return {
            'accept': 'application/json',
            'authorization': f'Bearer {self.auth_client.get_access_token()}'
        }

class TripsModule:
    def __init__(self, core: TelematicsCore):
        self.core = core  # Ensuring naming consistency

    @property
    def trips(self):
        return Trips(self.core.auth_client)


class Trips(BaseTrips):

    def get_list_trips(self, user_id,
                       start_date=None, end_date=None,
                       start_date_timestamp_sec=None, end_date_timestamp_sec=None,
                       include_details=True, include_statistics=False,
                       include_scores=False, include_related=False,
                       tags_included=None, tags_included_operator=None,
                       tags_excluded=None, tags_excluded_operator=None,
                       locale="EN", unit_system=None,
                       vehicles=None, sort_by=None,
                       limit=None, page=None):
        """
        Retrieves trip details for a specific user, with support for automatic pagination.

        :return: Aggregated trip details in JSON format.
        """
        # Set default limit and adjust if necessary
        if limit is None:
            limit = 14
        elif limit > 14:
            print("[NOTIFICATION] Limit was adjusted to 14")
            limit = 14

        if sort_by is None:
            sort_by = "StartDateUtc_Desc"

        if unit_system not in ["Si", "km", "ml", "Imperial"]:
            print("[NOTIFICATION] Invalid unit_system provided. Defaulting to 'Si'.")
            unit_system = "Si"
        if unit_system == "km":
            unit_system = "Si"
        elif unit_system == "ml":
            unit_system = "Imperial"

        url = f"{self.BASE_URL}"

        payload = {
            "Identifiers": {
                "UserId": user_id
            },
            "IncludeDetails": include_details,
            "IncludeStatistics": include_statistics,
            "IncludeScores": include_scores,
            "IncludeRelated": include_related,
            "Locale": locale,
            "UnitSystem": unit_system,
            "SortBy": sort_by,

        }

        if start_date and end_date:
            payload["StartDate"] = start_date
            payload["EndDate"] = end_date
        elif start_date_timestamp_sec and end_date_timestamp_sec:
            payload["StartDateTimestampSec"] = start_date_timestamp_sec
            payload["EndDateTimestampSec"] = end_date_timestamp_sec

        if tags_included is not None and not isinstance(tags_included, list):
            tags_included = [tags_included]

        if tags_excluded is not None and not isinstance(tags_excluded, list):
            tags_excluded = [tags_excluded]

        self._add_to_payload_if_exists(payload, "TagsIncluded", tags_included)
        self._add_to_payload_if_exists(payload, "TagsIncludedOperator", tags_included_operator)
        self._add_to_payload_if_exists(payload, "TagsExcluded", tags_excluded)
        self._add_to_payload_if_exists(payload, "TagsExcludedOperator", tags_excluded_operator)
        self._add_to_payload_if_exists(payload, "Vehicles", vehicles)

        # Initialize variables for pagination
        all_trips = []
        current_page = 1
        total_pages = 1  # Start with assumption there's at least one page

        if page is None:
            while current_page <= total_pages:
                payload["Paging"] = {
                    "Page": current_page,
                    "Count": limit,
                    "IncludePagingInfo": True
                }

                # Fetch current page of trips
                trips_response = self._fetch_trip_details(url, payload, limit)
                # Correctly access the trips using the .trips property of TripsResponse
                all_trips.extend(trips_response.trips)

                # Update paging information based on the current response
                paging_info = trips_response.paging_info
                current_page = paging_info.get("CurrentPage", current_page) + 1
                total_pages = paging_info.get("TotalPages", total_pages)
        else:
            payload["Paging"] = {
                "Page": page,
                "Count": limit,
                "IncludePagingInfo": True
            }

            # Fetch current page of trips
            trips_response = self._fetch_trip_details(url, payload, limit)
            # Correctly access the trips using the .trips property of TripsResponse
            all_trips.extend(trips_response.trips)

        final_response = {
            "Result": {
                "Trips": all_trips,
                "PagingInfo": {
                    "CurrentPage": current_page - 1,  # Adjust because of the loop increment
                    "TotalPages": total_pages
                }
            },
            "InitialPayload": payload  # Include for reference
        }

        return TripsResponse(final_response)

    def _fetch_trip_details(self, url, payload, limit=None):
        try:
            response = self.auth_client.post_with_retry(url, headers=self._get_headers(), json=payload)

            if response.status_code == 200:
                # Successful response, parse JSON
                data = response.json()
                return TripsResponse(data)

            # For any non-200 response, return an empty TripsResponse with an additional status attribute
            return TripsResponse({"status": response.status_code, "content": response.text})

        except requests.exceptions.RequestException as e:
            print(f"Request error: {e}")
            return TripsResponse({"status": "error", "message": str(e)})

        except json.JSONDecodeError as e:
            print(f"JSON decode error: {e}")
            print(f"Response content: {response.text}")
            return TripsResponse({"status": "error", "message": "JSON decode error"})

        

    def _add_to_payload_if_exists(self, payload, key, value):
        if value:
            payload[key] = value

class TripsResponse:
    def __init__(self, data):
        self.data = data if isinstance(data, dict) else {}

    @property
    def result(self):
        # Safeguard in case 'Result' is not a dictionary
        return self.data.get('Result', {}) if isinstance(self.data.get('Result', {}), dict) else {}

    @property
    def trips(self):
        # This already safeguards against 'Result' not being a dictionary and 'Trips' not being a list
        trips = self.result.get('Trips', [])
        return trips if isinstance(trips, list) else []

    @property
    def statistics(self):
        # Similar safeguard as in 'trips'
        trip = self.result.get('Trip', {})
        return trip.get('Statistics', {}) if isinstance(trip, dict) else {}

    @property
    def details(self):
        # Since 'Data' is nested within 'Trip', the same pattern applies here
        return self.result.get('Trip', {}).get('Data', {})

    @property
    def transporttype(self):
        # Assuming 'TransportType' is always a dictionary or should be an empty one if not present
        return self.details.get('TransportType', {})

    @property
    def scores(self):
        # Assuming 'Scores' is always a dictionary or should be an empty one if not present
        return self.result.get('Trip', {}).get('Scores', {})

    @property
    def events(self):
        # Assuming 'Events' is always a list or should be an empty one if not present
        events = self.result.get('Trip', {}).get('Events', [])
        return events if isinstance(events, list) else []

    @property
    def waypoints(self):
        # Assuming 'Waypoints' is always a list or should be an empty one if not present
        waypoints = self.result.get('Trip', {}).get('Waypoints', [])
        return waypoints if isinstance(waypoints, list) else []

    @property
    def paging_info(self):
        # Assuming 'PagingInfo' is always a dictionary or should be an empty one if not present
        return self.result.get('PagingInfo', {})

    @property
    def status(self):
        # 'Status' should be a singular value, not a list or dict
        return self.data.get('Status')

    @property
    def datetime(self):
        # Assuming 'Data' is always a dictionary or should be an empty one if not present
        return self.data.get('Data', {})

    @property
    def trip_id(self):
        # Assuming 'Id' is a singular value, not a list or dict
        return self.result.get('Id')

    @property
    def full_response(self):
        # The full response is just the data provided to the instance
        return self.data
    
    def __iter__(self):
        for item in self.data:
            yield item

    def __str__(self):
        return json.dumps(self.data, indent=4)
    



    # Add at the bottom of statistics.py
def DamoovAuth(email, password):
    auth_client = TelematicsAuth(email, password)
    return Trips(auth_client)




# damoov_admin/__init__.py
from . import auth
from .statistics import Statistics
from .core import TelematicsCore
from .trips import Trips
from .users import Users
from .engagement import Engagement
from .accounts import Accounts
